// **********************************************************************
//
// Copyright (c) 2008-2023 VCI-Tech, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described the
// ICE_LICENSE file included this distribution.
//
// **********************************************************************

#pragma once

#include "01-common.ice"

[["java:package:com.vci.corba"]]
["cs:namespace:com.vci.corba"]

// module com {
// module vci {
// module corba {
module omd {

	// module ddl{
	// 	interface DDLService{
	// 		bool generatorODDLForBt(string apContent, string btContent);
	// 		bool generatorSDDLForBt(string apContent, string btContent);
	// 		bool generatorODDLForLt(string apContent, string ltContent);
	// 		bool generatorSDDLForLt(string apContent, string ltContent);
		
	// 		//创建, 修改, 删除业务类型Table
	// 		bool executeUpdateOracle(string sql);
	// 		bool executeUpdateSQLServer(string sql);
	// 		//批量执行sql
	// 		bool batchExecuteUpdateOracle(common::data::StringArray sqlArray);
	// 		bool batchExecuteUpdateSQLServer(common::data::StringArray sqlArray);
			
	// 		//记录修改Table DDL(alter table t add(column type)
	// 		bool alterTableDDLOracle(string sql);
	// 		bool alterTableDDLSQLServer(string sql);
			
	// 		//检查业务类型, 链接类型在数据库中是否已有数据对象
	// 		bool hasInstanceOralce(string name);
	// 		bool hasInstanceSQLServer(string name);
	// 	};
	// };

	// 枚举类型管理
	module etm{
	
		struct EnumChild{
			string name;
			string value;
			string description;
		};
		sequence<EnumChild> EmChildren;
		struct EnumItem{
			string oid;
			string ts;
			string creator;
			long createTime;
			string modifier;
			long modifyTime;
			string name;
			string label;
			string type;
			long length;
			EmChildren children;
		};
		sequence<EnumItem> EnumItemArray;
		
		interface EnumService{
			//增加枚举
			bool addEmItem(EnumItem ei)throws common::VCIError;
			//修改枚举
			bool modifyEmItem(EnumItem ei)throws common::VCIError;
			//删除枚举
			bool deleteEmItem(EnumItem ei)throws common::VCIError;
			bool deleteEmItems(EnumItemArray eis)throws common::VCIError;
			//查询枚举
			EnumItemArray getEmItems(string filter, long start, long rows)throws common::VCIError;
			//检查枚举名是否存在
			bool checkRowIsExists(string name)throws common::VCIError;
			//根据枚举类型查询枚举
			EnumItemArray getEmItemsByType(string type)throws common::VCIError;
			//根据枚举名字查询枚举
			EnumItem getEmItemByName(string name)throws common::VCIError;
		};
	};
	
	// 属性定义管理接口
	module atm{
		struct AttribItem{
			string oid;
			string ts;
			string creator;
			long createTime;
			string modifier;
			long modifyTime;
			string name;
			string label;
			string description;
			string vtDataType;
			string defValue;
			string rage;
			string other;
		};
		sequence<AttribItem> AttribItemArray;

		interface AttPoolService{
			//添加属性项
			bool addAttribItem(AttribItem attr)throws common::VCIError;
			//修改属性项
			bool modifyAbItem(AttribItem attr)throws common::VCIError;
			//删除参数列表中的属性项
			bool deleteAbItems(AttribItemArray attrs)throws common::VCIError;
			bool deleteAbItem(AttribItem attr)throws common::VCIError;
			//查询属性项
			AttribItemArray getAttribItems(string filter, long start, long rows)throws common::VCIError;
			//根据属性名列表查询属性项
			AttribItemArray getAttribItemsByNames(common::data::StringArray abNameArray)throws common::VCIError;
			//查询不在参数列表中的属性项
			AttribItemArray getAttribItemsOutNames(common::data::StringArray abNameArray,string text)throws common::VCIError;
			//检查要插入的记录是否存在
			bool checkRowIsExists(string name)throws common::VCIError;
			//根据属性名获取属性
			AttribItem getAttribItemByName(string abName)throws common::VCIError;				
			//根据属性名获取属性数据类型
			string getAttribItemDataType(string abName)throws common::VCIError;
			//提供AP数据文件内容
			string getAPData()throws common::VCIError;
			//获取使用指定枚举名的属性名列表
			common::data::StringArray getAPNamesByEMName(string emName)throws common::VCIError;
		};
	};
	// 状态管理接口
	module stm{

		struct StatePool{
			string oid;
			string ts;
			string creator;
			long createTime;
			string modifier;
			long modifyTime;
			string id;
			string name;
			string tag;
			string imagePath;
			string description;
		};
		sequence<StatePool> StatePoolArray;
		
		interface StatePoolService{
			//增加statePool
			bool addStatePool(StatePool state)throws common::VCIError;
			//修改statePool
			bool modifyStatePool( StatePool state)throws common::VCIError;
			//删除statePool
			bool deleteStatePool( StatePool state)throws common::VCIError;
			//删除statePools
			bool deleteStatePools(StatePoolArray states)throws common::VCIError;
			//获取所有的statePool
			StatePoolArray getStatePools()throws common::VCIError;
			//获取statePool
			StatePool getStatePool(string name)throws common::VCIError;
			
			common::data::StringArray getImagePaths()throws common::VCIError;
		};
	};

	// 生命周期模型管理
	module lcm{
		struct Bound{
			string id;
			string name;
			string cellx;
			string celly;
			string cellw;
			string cellh;
			string cellicon;
		};
		sequence<Bound> BoundArray;
		struct TransitionVOEvent{
			string id;
			string name;
		};
		sequence<TransitionVOEvent> TransitionVOEventArray;
		struct TransitionVO{
			string id;
			string source;
			string destination;
			string connect;
			TransitionVOEventArray transitionVOEvents;
		};
		sequence<TransitionVO> TransitionVOArray;
		struct LifeCycle{
			string oid;
			string ts;
			string creator;
			long createTime;
			string modifier;
			long modifyTime;
			string id;
			string name;
			string tag;
			string description;
			string startState;
			BoundArray bounds;
			TransitionVOArray routes;
		};
		sequence<LifeCycle> LifeCycleArray;
		
		struct PropertyInfo{
			string property;
			string value;
		};
		sequence<PropertyInfo> PropertyInfoList;
		//事务历史记录
		struct TransationHistoryRecord{
			string id;
			string name;
			TransitionVO transationVO;
			string btmName;
			string transationTime;
			string userName;
		};
		sequence<TransationHistoryRecord> TransationHistoryRecords;
		
		interface LifeCycleService{
			//增加生命周期
			bool addLifeCycle(LifeCycle lc)throws common::VCIError;
			//修改生命周期
			bool modifyLifeCycle(LifeCycle lc)throws common::VCIError;
			//删除生命周期
			bool deleteLifeCycle(LifeCycle lc)throws common::VCIError;
			//获取所有的生命周期
			LifeCycleArray getLifeCycles()throws common::VCIError;
			//获取指定的生命周期
			LifeCycle getLifeCycle(string name)throws common::VCIError;
			string getLifeCycleEventPath()throws common::VCIError;
			string getLifeCycleEventViewPath()throws common::VCIError;
			string getLifeCycleEventViewSavePath()throws common::VCIError;
			
			string getLifecycleEventPath2() throws common::VCIError;
			
			//记录事务历史
			bool recordTransitionHistory(TransationHistoryRecord thr)throws common::VCIError;
			
			//执行事务
			//void excuteTransitionEvents (TransitionVO transitionVO)throws common::VCIError;
			
			//获取事件key
			common::data::StringArray getLCEventKeys()throws common::VCIError;
			//获取事件value
			string getLCEventValueByKey(string key)throws common::VCIError;
			//删除生命周期
			bool deleteLifeCycles(LifeCycleArray lcs)throws common::VCIError;
		};
	};
	// 版本规则管理接口
	module vrm {
		struct VersionRule{
			string oid;
			string ts;
			string creator;
			long createTime;
			string modifier;
			long modifyTime;
			string id;
			string name;
			string tag;
			string jumpCharacter;
			string initialValue;
			string stepLength;
			string prefixion;
			string suffix;
			string description;
			string associated;
		};
		sequence<VersionRule> VersionRuleArray;

		interface VersionRuleService{
			//增加VersionRule
			bool addVersionRule(VersionRule vr)throws common::VCIError;
			//修改VersionRule
			bool modifyVersionRule( VersionRule vr)throws common::VCIError;
			//删除VersionRule
			bool deleteVersionRule( VersionRule vr)throws common::VCIError;
			//获取所有的VersionRules
			VersionRuleArray getVersionRules()throws common::VCIError;
			//删除VersionRules
			bool deleteVersionRules(VersionRuleArray vrs)throws common::VCIError;
			//获取指定的VersionRules
			VersionRule getVersionRule(string name)throws common::VCIError;
		};
	};

	// 业务类型管理接口
	module btm{
		// 业务类型
		struct BtmItem{
			string oid;
			long ts;
			string creator;
			long createTime;
			string modifier;
			long modifyTime;
			string name;
			string label;
			string description;
			bool isAbstract;
			string fName;
			string implClass;
			string shape;
			string lifeCycle;
			common::data::StringArray lifeCycles;
			string imageName;
			short revLevel;
			string revRuleName;
			//是否手工输入版本号规则
			bool revInput;
			string delimiter;
			short verRuleName;
			common::data::StringArray apNameArray;
		};
		sequence<BtmItem>  BtmItemArray;

		struct BtmAndApName{
			string btmName;
			common::data::StringArray apName;
		};
		sequence<BtmAndApName> BtmAndApNameArray;

		interface BTMService{
			//获取全部业务类型
			BtmItemArray getAllBtmItem(string filter) throws common::VCIError;
			//获取当前业务类型的子类型
			BtmItemArray getChildrenBtms(string btmName) throws common::VCIError;
			//获取指定的业务类型
			BtmItem getBtmItemByName(string btmName)throws common::VCIError;
			//获取属性名和对应的业务类型名
			//BtmAndApNameArray getBtmAndApNameArray(string btmName)throws common::VCIError;
			//增加业务类型
			bool addBtmItem(BtmItem bt)throws common::VCIError;
			//bool addBtmItemNoCache(BtmItem bt)throws common::VCIError;
			//修改业务类型
			bool updateBtmItem(BtmItem bt)throws common::VCIError;
			//删除业务类型
			bool deleteBtmItem(BtmItem bt)throws common::VCIError;
			//bool deleteBtmItemNoCache(BtmItem bt)throws common::VCIError;
			// 修改业务类型属性字段
			bool modifyBTAttribute(string btName, string attrName)throws common::VCIError;
			//检查要插入的记录是否存在
			bool checkRowIsExists(string name)throws common::VCIError;
			//获取业务类型的属性名列表
			common::data::StringArray getBtmApNameArray(string btmName)throws common::VCIError;
		
			// 获取指定类型的所有属性
			atm::AttribItemArray getAttributes(string btName)throws common::VCIError;
			//提供BTM数据文件内容
			string getBTMData()throws common::VCIError;
			//根据属性名获取使用该属性名的全部业务类型名
			common::data::StringArray getBTMNamesByAPName(string apName)throws common::VCIError;
			//根据版本规则名获取使用该版本规则的全部业务类型名
			common::data::StringArray getBTMNamesByVerName(string verName)throws common::VCIError;
			//根据生命周期名获取使用该生命周期的全部业务类型名
			common::data::StringArray getBTMNamesByLCyName(string lCyName)throws common::VCIError;
			//获取服务端图片路径后缀
			common::data::StringArray getImagePaths()throws common::VCIError;
			//获取业务类型的子类型名
			common::data::StringArray getChildrenNames(string btmName)throws common::VCIError;
			//业务类型与数据库一致性检查
			common::data::StringArray btmConsistencyCheck()throws common::VCIError;
			//执行修复的sql
			common::data::StringArray executeRepair(common::data::StringArray sqlArray)throws common::VCIError;
		
			// 添加索引
			bool addIndex(string typeName, common::data::StringArray indexAttr) throws common::VCIError;
			// 删除索引
			bool dropIndex(string typeName, common::data::StringArray indexAttr) throws common::VCIError;
			// 创建业务对象表
			bool createTable(string btmName) throws common::VCIError;
			//创建业务类型视图
			bool createView()throws common::VCIError;
			//清空表中数据
			bool truncateTable(string btName)throws common::VCIError;
			//清空表中数据
			bool deleteTable(string btName)throws common::VCIError;
			//清空业务表， 业务类型
			bool deleteBtsAndTables(BtmItemArray bts)throws common::VCIError;
			//清空业务类型
			bool deleteBtmItems(BtmItemArray bts)throws common::VCIError;
			//获取表中fields中不能 删除的字段
			common::data::StringArray getUnRemovableFields(string btName, common::data::StringArray fields)throws common::VCIError;
			//若表tableName不存在，则用createSql创建表
			bool checkTable(string tableName, string createSql)throws common::VCIError;
		
			// 查询指定对象是否存在实例数据
			bool hasData(string btName)throws common::VCIError;
			
			// 获取业务类型系统属性名称
			common::data::StringArray getSysAttributes();
			
			//  获取业务类型系统属性
			atm::AttribItemArray getSysAttribItems();
			
			// 获取业务类型名称最大长度
			int getBTNameMaxLength();
		};
	};

	// 链接类型管理接口
	module ltm {
		
		struct LinkType{
			string oid;
			long ts;
			string creator;
			long createTime;
			string modifier;
			long modifyTime;
			string id;//链接类型唯一标识
			string name;//链接类型名称
			string tag;//链接类型标签
			string description;//描述
			common::data::StringArray btmItemsFrom;//From端业务类型名称集合
			common::data::StringArray btmItemsTo;//to端业务类型名称集合
			string primitivesFrom;//from端主类型
			string primitivesTo;//to端主类型
			string relationFrom;//from端关系
			string relationTo;//to端关系
			string relation;
			string implClass;//实现类
			string shape;//形状
			common::data::StringArray attributes;//属性名称集合
		};
		sequence<LinkType> LinkTypeArray;
		
		interface LinkTypeService{
			/**添加链接类型**/
			bool addLinkType(LinkType lt)throws common::VCIError;
			/**修改链接类型**/
			bool modifyLinkType(LinkType lt)throws common::VCIError;
			/**删除链接类型**/
			bool deleteLinkType(LinkType lt)throws common::VCIError;
			/**获取所有链接类型集合**/
			LinkTypeArray getLinkTypes()throws common::VCIError;
			/**获取指定的链接类型**/
			LinkType getLinkType(string name)throws common::VCIError;
		
			// 获取指定类型的所有属性
			atm::AttribItemArray getAttributes(string ltName)throws common::VCIError;
			/**(对外)提供链接类型的数据文件数据**/
			string getLTData()throws common::VCIError;
			/**获取使用指定属性的链接类型名称**/
			common::data::StringArray getLTNamesByAPName(string apName)throws common::VCIError;
			//业务类型与数据库一致性检查
			common::data::StringArray linkTypeConsistencyCheck()throws common::VCIError;
			//执行修复的sql
			common::data::StringArray executeRepair(common::data::StringArray sqlArray)throws common::VCIError;
		
			// 修改链接类型属性字段
			bool modifyLTAttribute(string btName, string attrName)throws common::VCIError;
		
			// 添加索引
			bool addIndex(string typeName, common::data::StringArray indexAttr) throws common::VCIError;
			// 删除索引
			bool dropIndex(string typeName, common::data::StringArray indexAttr) throws common::VCIError;
			
			// 创建链接类型数据表
			bool createTable(string ltName)throws common::VCIError;
		
			//创建链接类型查询全部To/From端业务类型的视图
			bool createView()throws common::VCIError;
			
			//清空表中数据
			bool truncateTable(string ltName)throws common::VCIError;
			
			//清空链接表， 链接类型
			bool deleteLtsAndTables(LinkTypeArray lts)throws common::VCIError;
			//清空链接类型
			bool deleteLinkTypes(LinkTypeArray lts)throws common::VCIError;
			//链接类型获取根据业务类型名及方向
			LinkTypeArray getLinkTypeByBtmName(string btmName, string direction)throws common::VCIError;
		
			// 查询指定链接对象是否存在实例数据
			bool hasData(string ltName)throws common::VCIError;
			
			// 获取系统属性名称
			common::data::StringArray getSysAttributes();
						
			//  获取业务类型系统属性
			atm::AttribItemArray getSysAttribItems();
			
			// 获取链接类型名称最大长度
			int getLTNameMaxLength();
		};
	};
	
		
	// 对象类型索引管理
	module tim {
			
		/**业务类型索引**/
		struct TypeIndexDef {
			string oid;			// 索引OID
			string name;		// 索引名称
			string typeName;	// 类型名称
			short  type;		// 业务类型：1；链接类型：2；
			string attributes;	// 索引属性名称，多个属性使用";"分割
			string description;	// 描述
			string creator;		// 创建者
			long createTime;	// 创建时间
			string modifier;		// 创建者
			long modifyTime;	// 创建时间
		};
		sequence<TypeIndexDef> TypeIndexDefArray;
	
		interface TypeIndexService {
					
			// 获取业务类型索引
			TypeIndexDefArray getBTIndexDefs(string btName) throws common::VCIError;
			
			// 获取业务类型索引
			TypeIndexDefArray getLTIndexDefs(string ltName) throws common::VCIError;
			
			// 保存索引
			void saveIndexDef(TypeIndexDef def) throws common::VCIError;
			// 批量保存索引
			void saveIndexDefs(TypeIndexDefArray defs) throws common::VCIError;
			
			// 更新索引
			void updateIndexDef(TypeIndexDef def) throws common::VCIError;
			// 批量更新索引
			void updateIndexDefs(TypeIndexDefArray defs) throws common::VCIError;

			// 删除索引
			void delIndexDef(string oid) throws common::VCIError;

			// 批量删除索引
			void delIndexDefs(common::data::StringArray oids) throws common::VCIError;
		};
	};
	
	
	module qtm {
			
		struct QTD{
			string name;
			string creator;
			long createTime;
			string btmName;
			string linkTypeName;
			common::data::StringArray abNames;
		};
		sequence<QTD> QTDArray;

		struct QTInfo{
			string qtName;
			string btmName;
			string creator;
			long createTime;
			//0: 普通; 1: 高级
			short levelFlag;
			//记录qtUI结构
			string qtUIText;
			//可将qtText通过XML转化成qt
			string qtText;
		};
		sequence<QTInfo> QTInfoArray;
			
				
		interface QTDService{
			//查询名字是否已经被使用
			bool isExistsQTD(string name)throws common::VCIError;
			//增加QTD
			bool addQTD(QTD qtdObj)throws common::VCIError;
			//修改QTD
			bool modifyQTD(QTD qtdObj)throws common::VCIError;
			//删除QTD
			bool deleteQTD(string qtdName)throws common::VCIError;
			//获取某个业务类型下的所有QTD
			QTDArray getBizTypeQTDs(string btmName)throws common::VCIError;
			//获取某个链接类型下的所有QTD
			QTDArray getLinkTypeQTDs(string linkTypeName)throws common::VCIError;
			//获取QTD
			QTD getQTDByName(string name)throws common::VCIError;
			//获取所有的QTDS
			QTDArray getAllQTDs()throws common::VCIError;
			//获取所有业务类型的QTDS
			QTDArray getAllBizTypeQTDs()throws common::VCIError;
			//获取所有链接类型的QTDS
			QTDArray getAllLinkTypeQTDs()throws common::VCIError;
			
			//检查名字是否存在
			bool isExistsQT(string name)throws common::VCIError;
			//根据查询模板名获取查询模板
			QTInfo getQT(string qtName)throws common::VCIError;
			//从数据库获取当前业务类型的所有查询模板
			QTInfoArray getObjTypeQTs(string btmName)throws common::VCIError;
			//从数据库获取所有的查询模板
			QTInfoArray getAllQTs()throws common::VCIError;
			//保存QT
			bool saveQT(QTInfo qtw)throws common::VCIError;
			//删除查询模板
			bool deleteQTs(common::data::StringArray qtNames)throws common::VCIError;
			//更新QT将qtUIText和qtText变更为CLOB字段
			bool updateQT()throws common::VCIError;
		};

	
	}

	module data {
		//system conf info
		struct SystemModelData {
			vrm::VersionRuleArray versionRules;
			stm::StatePoolArray states;
			lcm::LifeCycleArray lifecycles;
			ltm::LinkTypeArray linkTypes;
			etm::EnumItemArray enumItems;
			atm::AttribItemArray attrItems;
			btm::BtmItemArray btmItems;
		};

		/**属性对象**/
		struct AttributeValue {
			string attrName; //属性对应数据库表的列名称
			string attrVal; //属性值
		};
		sequence<AttributeValue> AttributeValueList;

		/**业务对象**/
		struct BusinessObject {
			string oid;
			string revisionid;
			string nameoid;
			string btName;
			bool isLastR;
			bool isFirstR;
			bool isLastV;
			bool isFirstV;
			string creator;
			long createTime;
			string modifier;
			long modifyTime;
			string revisionRule;
			string versionRule;
			short revisionSeq;
			string revisionValue;
			short versionSeq;
			string versionValue;
			string lctId;
			string lcStatus;
			long ts;
			string id;
			string name;
			string description;
			string owner;
			//string checkinBy;
			//string checkinTime;
			//string checkoutBy;
			//string checkoutTime;
			string fromVersion;
			AttributeValueList newAttrValList;
			AttributeValueList hisAttrValList;
		};//
		sequence<BusinessObject> BusinessObjectList;

		/**链接对象**/
		struct LinkObject{
			string oid;
			string creator;
			long createTime;
			string modifier;
			long modifyTime;
			string fromOid;
			string fromRevOid;
			string fromNameOid;
			string fromBTName;
			string toOid;
			string toRevOid;
			string toNameOid;
			string toBTName;
			long ts;
			string ltName;
			AttributeValueList newAttrValList;
			AttributeValueList hisAttrValList;
		};
		sequence<LinkObject> LinkObjectList;


		/**版本数据对象**/
		struct RevisionDataInfo {
			string revisionVal;
			short revisionSeq;
		};

		/**版次数据对象**/
		struct VersionDataInfo {
			string versionVal;
			short versionSeq;
		};

	};
	


	// 对象模型定义服务
	interface OMDService {
		void test();
		
		//根据传入的业务对象名称，获取对象建模的所有数据（参数为空时，获取所有建模数据）
		data::SystemModelData getSysModelData(common::data::StringArray boTypes) throws common::VCIError;

		// 获取DDL服务
		//ddl::DDLService* getDDLService() throws common::VCIError;

		// 获取枚举类型服务
		etm::EnumService* getEnumService() throws common::VCIError;

		// 获取属性管理服务
		atm::AttPoolService* getAttributeService() throws common::VCIError;

		// 获取生命周期状态服务
		stm::StatePoolService* getStateService() throws common::VCIError;

		// 获取生命周期模型服务
		lcm::LifeCycleService* getLifeCycleService() throws common::VCIError;

		// 获取版本规则类型服务
		vrm::VersionRuleService* getVerRuleService() throws common::VCIError;

		// 获取业务类型服务
		btm::BTMService* getBTMService() throws common::VCIError;

		// 获取链接类型服务
		ltm::LinkTypeService* getLinkTypeService() throws common::VCIError;
		
		// 获取类型索引管理服务
		tim::TypeIndexService* getTypeIndexService() throws common::VCIError;

		// 获取查询模板定义服务
		qtm::QTDService* getQTDService() throws common::VCIError;

	}
}//common			
// }}}